/*
 *	avprog.c: routines to handle AV progs
 *
 * 	reads config file, provides functions to setup and run AV progs and
 *	to update AV signatures as defined by config file
 *
 *	(C) Oliver Diedrich 2004 odi@ct.heise.de
 *      Licensed under the GNU Public license.
 */

 
 
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <limits.h>

#include "ctcd.h"

/* only needed when compiling stand alone for debugging purposes
#define wait_for_key() (void) getchar();
#define clear() write(STDOUT_FILENO, CLEAR, strlen(CLEAR));
*/


int has_string(char *str)
/* checks if str[] contains %s */
/* 1: yes, 0: no */
{
  char *c = str;
  if (!str) return 0;
  while(*c>0)
  {
    if ('%' == *c)
    {
      c++;
      if ('s' == *c) return 1;
    }
    c++;
  }
  return 0;
} /* has_string() */



int read_config(avprog_t *avprog[], char **error)
/* reads AV prog configuration from AVPROG_CONFIG and puts it into avprog[] */
/* -1: error in config file or config file not found, else: # av progs */
/* in case of errors, *error is set to an error message */
{
  int i = 0;
  int n_avprog = 0;
  int j, status;
  FILE *f;
  char buf[1024];
  char *c;

  void line(char *from, char **to)
  /* copies string from *from to *to; memory for *to is allocated */
  {
    while (' ' == *from || ':' == *from || '\t' == *from) from++;
    if (*from)
    {
      if (from[0])
      {
        *to = malloc(strlen(from)+1);
        memset(*to, 0, strlen(from)+1);
        strcpy(*to, from);
      }
    }
  } /* line() */
  
  f = fopen(AVPROG_CONFIG, "r");
  if (!f) 
  {
    *error = malloc(30 + strlen(AVPROG_CONFIG));
    sprintf(*error, "%s kann nicht geffnet werden.", AVPROG_CONFIG);
    return -1;
  }

  while (fgets(buf, AVPROG_CONFIG_LINE_LENGHT, f))
  {
    c = buf;
    if ('#' == *c) continue;           /* comment */
    while (*c > '\0')                  /* ignore leading white spaces */
    {
      if (' ' == *c || '\t' == *c) c++;
      else break;
    }
    if (*c == '\0') continue;           /* empty line */
    if (*c == '\n') continue;

    if ('\n' == c[strlen(c)-1]) c[strlen(c)-1] = '\0';

    if ('[' == *c)                      /* header, taken as name of AV prog */
    {
      char *e = c++;
      avprog[n_avprog] = malloc(sizeof(avprog_t));
      i = n_avprog;
      n_avprog++;
      while ('\0' != *e && '\n' != *e && ']' != *e) e++;
      *e = '\0';
      if (e > c) 
      {
        avprog[i]->name = malloc(strlen(c)+1);
        strcpy(avprog[i]->name, c);
      }
      else avprog[i]->name = NULL;
      avprog[i]->setup = NULL;
      avprog[i]->wd = NULL;
      avprog[i]->update_net = NULL;
      avprog[i]->update_disk = NULL;
      avprog[i]->version = NULL;
      avprog[i]->check = NULL;
      avprog[i]->log = NULL;
      avprog[i]->selected = 1;
    }

    else                                /* keyword : statement */
    {
      if (!strncmp(c, "setup", 5))
      {
        c+=5;
        line(c, &avprog[i]->setup);
      }
      else if (!strncmp(c, "wd", 2))
      {
        c+=2;
        line(c, &avprog[i]->wd);
      }
      else if (!strncmp(c, "update_net", 10))
      {
        c+=10;
        line(c, &avprog[i]->update_net);
      }
      else if (!strncmp(c, "update_disk", 11))
      {
        c+=11;
        line(c, &avprog[i]->update_disk);
      }
      else if (!strncmp(c, "version", 7))
      {
        c+=7;
        line(c, &avprog[i]->version);
      }
      else if (!strncmp(c, "check", 5))
      {
        c+=5;
        line(c, &avprog[i]->check);
      }
      else if (!strncmp(c, "disinfect", 9))
      {
        c+=9;
        line(c, &avprog[i]->disinfect);
      }
      else if (!strncmp(c, "log", 3))
      {
        c+=3;
        line(c, &avprog[i]->log);
      }
      else 
      {
        *error = malloc(25 + strlen(c));
        if ('\n' == c[strlen(c)-1]) c[strlen(c)-1] = '\0';
        sprintf(*error, "'%s': fehlerhafte Zeile", c);
        return -1;
      }
    }

  } /* while (fgets()) */
  fclose(f);

  for (i=0; i<n_avprog; i++)
  {
    if (!avprog[i]->check)              /* no check routine: remove */
    {
      for (j=i+1; j<n_avprog; j++) avprog[j-1] = avprog[j];
      n_avprog--;
      i--;
    }
  }

  for (i=0; i<n_avprog; i++)            /* setup does not run: remove */
  {
    if (avprog[i]->setup)
    {
      status = system(avprog[i]->setup);
      if WEXITSTATUS(status)            /* setup error: remove from avprog[] */
      {
        for (j=i+1; j<n_avprog; j++) avprog[j-1] = avprog[j];
        n_avprog--;
        i--;
      }
    }
  }
  return(n_avprog);
} /* read_config() */



int av_version(avprog_t *avprog)
/* lets avprog print its version */
/* 0: error, 1: ok */
{
  char cwd[PATH_MAX];
  int status;

  if (!avprog->version) return 0;
  getcwd(cwd, PATH_MAX);
  chdir(avprog->wd);
  status = system(avprog->version);
  chdir(cwd);
  if WEXITSTATUS(status) return 0;
  else 
  {
    printf("\n");
    return 1;
  }
} /* av_version() */



int av_update_net(avprog_t *avprog, char *proxy)
/* updates signatures for avprog via internet */
/* 0: error, 1: ok */
{
  char cwd[PATH_MAX];
  int status;
  char str[4500];

  if (!avprog->update_net) return 0;
  clear();
  printf("\nJe nach Internet-Anbindung kann das Herunterladen der Signaturdateien\n");
  printf("fr %s einige Minuten dauern.\n\n", avprog->name);
  getcwd(cwd, PATH_MAX);
  chdir(avprog->wd);
  if (proxy)
  {
    sprintf(str, "export http_proxy=%s ; %s", proxy, avprog->update_net);
    status = system(str);
  }
  else status = system(avprog->update_net);
  chdir(cwd);
  if WEXITSTATUS(status) return 0;
  else return 1;
} /* av_udate_net() */


  
int av_update_disk(avprog_t *avprog, char *path)
/* updates signatures for avprog from path. If path is !NULL, it replaces */
/* "%s" in avprog.update_disk before executing */
/* 0: error, 1: ok */
{
  char cwd[PATH_MAX];
  int status;
  char *buf;

  if (!avprog->update_disk) return 0;
  getcwd(cwd, PATH_MAX);
  chdir(avprog->wd);
  if (path)                   /* insert path in avprog.update_disk */
  {
    if (has_string(avprog->update_disk))
    {
      buf = malloc(strlen(avprog->update_disk) + strlen(path) + 1);
      sprintf(buf, avprog->update_disk, path);
      status = system(buf);
      free(buf);
    }
    else status = system(avprog->update_disk);
  }
  else status = system(avprog->update_disk);
  chdir(cwd);
  if WEXITSTATUS(status) return 0;
  else return 1;
} /* av_update_disk() */



int av_check(avprog_t *avprog, char *path)
/* checks path for viruses. If path is !NULL, it replaces */
/* "%s" in avprog.check before executing */
/* 0: error, 1: ok */
{
  char cwd[PATH_MAX];
  int status;
  char *buf;

  if (!avprog->check) return 0;
  getcwd(cwd, PATH_MAX);
  chdir(avprog->wd);
  unlink(avprog->log);        /* each run's log file is saved separately */
  if (path)                   /* insert path into avprog.check */
  {
    if (has_string(avprog->check))
    {
      buf = malloc(strlen(avprog->check) + strlen(path) + 1);
      sprintf(buf, avprog->check, path);
      status = system(buf);
      free(buf);
    }
    else status = system(avprog->check);
  }
  else status = system(avprog->check);
  chdir(cwd);
  if WEXITSTATUS(status) return 0;
  else return 1;
} /* av_check() */



int av_disinfect(avprog_t *avprog, char *path)
/* disinfects files in path. If path is !NULL, it replaces */
/* "%s" in avprog.disinfect before executing */
/* 0: error, 1: ok */
{
  char cwd[PATH_MAX];
  int status;
  char *buf;

  if (!avprog->disinfect) return 0;
  getcwd(cwd, PATH_MAX);
  chdir(avprog->wd);
  unlink(avprog->log);        /* each run's log file is saved separately */
  if (path)                   /* insert path into avprog.check */
  {
    if (has_string(avprog->disinfect))
    {
      buf = malloc(strlen(avprog->disinfect) + strlen(path) + 1);
      sprintf(buf, avprog->disinfect, path);
      status = system(buf);
      free(buf);
    }
    else status = system(avprog->disinfect);
  }
  else status = system(avprog->disinfect);
  chdir(cwd);
  if WEXITSTATUS(status) return 0;
  else return 1;
} /* av_disinfect() */



void execute(avprog_t *avprog[], int n_avprog, int debug)
/* NEEDED ONLY for quick config file testing */
{
  char cwd[PATH_MAX];
  int i, status;
  for (i=0; i<n_avprog; i++) 
  {
    printf("%s\n\n", avprog[i]->name);
    if (debug) printf("Setup: %s\n", avprog[i]->setup);
    status = system(avprog[i]->setup);
    if WEXITSTATUS(status)
      printf("%s:\nStatus %d, WEXITSTATUS(status): %d\n\n", 
             avprog[i]->setup, status, WEXITSTATUS(status));

    if (debug) printf("wd: %s\n", avprog[i]->wd);
    if (debug) printf("log: %s\n", avprog[i]->log);
    getcwd(cwd, PATH_MAX);
    chdir(avprog[i]->wd);

    if (debug) printf("Update_disk: %s\n", avprog[i]->update_disk);
    status = system(avprog[i]->update_disk);
    if WEXITSTATUS(status)
      printf("%s:\nStatus %d, WEXITSTATUS(status): %d\n\n", 
             avprog[i]->update_disk, status, WEXITSTATUS(status));

    if (debug) printf("Update_net: %s\n", avprog[i]->update_net);
    status = system(avprog[i]->update_net);
    if WEXITSTATUS(status)
      printf("%s:\nStatus %d, WEXITSTATUS(status): %d\n\n", 
             avprog[i]->update_net, status, WEXITSTATUS(status));

    if (debug) printf("Version: %s\n", avprog[i]->version);
    status = system(avprog[i]->version);
    if WEXITSTATUS(status)
      printf("%s:\nStatus %d, WEXITSTATUS(status): %d\n\n", 
             avprog[i]->version, status, WEXITSTATUS(status));

    if (debug) printf("check: %s\n", avprog[i]->check);
    status = system(avprog[i]->check);
    if WEXITSTATUS(status)
      printf("%s:\nStatus %d, WEXITSTATUS(status): %d\n\n", 
             avprog[i]->check, status, WEXITSTATUS(status));

    chdir(cwd);
  }
} /* execute() */




int Xmain()
/* for debugging purposes only ... */

{
  avprog_t *avprog[MAX_AVPROG];
  int i;
  char *err;
  int n_avprog = read_config(avprog, &err);

  if (-1 == n_avprog)
  {
    printf("Fehler beim Lesen vpn %s:\n%s\n", AVPROG_CONFIG, err);
    free(err);
    return 1;
  }
  else
  {
    printf("%d AV-Programm(e) gelesen:\n", n_avprog);
/*    execute(avprog, n_avprog, 0); */
    for (i=0; i<n_avprog; i++)
    {
      printf("\n%s (wd is %s, log name %s)\n", avprog[i]->name,
             avprog[i]->wd, avprog[i]->log);
      if (!av_version(avprog[i])) 
       printf(">>> ERROR %s: av_version()\n", avprog[i]->name);

      printf("\n>>> running '%s'\n", avprog[i]->update_disk);
      if (!av_update_disk(avprog[i], NULL))
       printf(">>> ERROR %s: av_update_disk()\n", avprog[i]->name);

      printf("\n>>> running '%s'\n", avprog[i]->update_net);
      if (!av_update_net(avprog[i], NULL))
       printf(">>> ERROR %s: av_update_net()\n", avprog[i]->name);

      if (!av_check(avprog[i], "/KNOPPIX"))
       printf(">>> ERROR %s: av_check()\n", avprog[i]->name);
    }
    return 0;
  }
}
