/*
 *	sysinfo.c: gathers and displays system and network information
 *
 *      uses parts of DMI decode rev 1.2
 *	(C) 2000,2001 Alan Cox <alan@redhat.com>
 *
 *      reads some info from /proc files, asks proxy settings
 *      (C) 2002, 2003, 2004, 2005 Oliver Diedrich
 *      
 *	Licensed under the GNU Public license.
 */

#define _LARGEFILE_SOURCE
#define _LARGEFILE64_SOURCE
#define _FILE_OFFSET_BITS 64


#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "ctcd.h"
#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>


typedef unsigned char u8;
typedef unsigned short u16;
typedef unsigned int u32;


struct dmi_header
{
	u8	type;
	u8	length;
	u16	handle;
};

static char *dmi_string(struct dmi_header *dm, u8 s)
{
	u8 *bp=(u8 *)dm;
	if (!s) return "";
	
	bp+=dm->length;
	while(s>1)
	{
		bp+=strlen(bp);
		bp++;
		s--;
	}
	return bp;
}


extern int check[];


char *sysinfo_mkstr(char str[])
/* allocates memory for string str, copies str to it, and removes '\n' and */
/* blanks at end */
/* NULL: error or str is empty, else: copied string */
{
  char *c;
  int l;

  if (!str) return NULL;
  l = strlen(str);
  if (!l) return  NULL;
  c = malloc(l+1);
  if (!c) return NULL;

  memset(c, 0, l+1);
  strncpy(c, str, l);
  l--;
  while ( ('\n' == c[l]) || (' ' == c[l]) )
  {
    c[l] = '\0';
    l--;
    if (0 == l) break;
  }
  if ('\0' == c[0]) 
  {
    free(c);
    return NULL;
  }
  else return c;
} /* sysinfo_mkstr() */



static char *dmi_chassis_type(u8 code)
{
	static char *chassis_type[]={
		"",
		"Other",
		"Unknown",
		"Desktop",
		"Low Profile Desktop",
		"Pizza Box",
		"Mini Tower",
		"Tower",
		"Portable",
		"Laptop",
		"Notebook",
		"Hand Held",
		"Docking Station",
		"All in One",
		"Sub Notebook",
		"Space-saving",
		"Lunch Box",
		"Main Server Chassis",
		"Expansion Chassis",
		"SubChassis",
		"Bus Expansion Chassis",
		"Peripheral Chassis",
		"RAID Chassis",
		"Rack Mount Chassis",
		"Sealed-case PC",
	};
	code &= ~0x80;
	
	if(code>0x18)
		return "";
	return chassis_type[code];
	
}



static void dmi_table(int fd, u32 base, int len, int num, sysinfo_t *si)
{
	char *buf=malloc(len);
	struct dmi_header *dm;
	u8 *data;
	int i=0;
	
	if(lseek(fd, (long)base, 0)==-1)
	{
		perror("dmi: lseek");
		return;
	}
	if(read(fd, buf, len)!=len)
	{
		perror("dmi: read");
		return;
	}
	data = buf;
	while(i<num)
	{
		dm=(struct dmi_header *)data;
		
		switch(dm->type)
		{
		case  0:
			si->bios_vendor = sysinfo_mkstr(dmi_string(dm, data[4]));
			si->bios_version = sysinfo_mkstr(dmi_string(dm, data[5]));
			si->bios_release = sysinfo_mkstr(dmi_string(dm, data[8]));
			break;
			
		case 1:
			si->sys_vendor = sysinfo_mkstr(dmi_string(dm, data[4]));
			si->sys_product = sysinfo_mkstr(dmi_string(dm, data[5]));
			si->sys_serial = sysinfo_mkstr(dmi_string(dm, data[7]));
			break;
			
		case 2:
			si->board_vendor = sysinfo_mkstr(dmi_string(dm, data[4]));
			si->board_product = sysinfo_mkstr(dmi_string(dm, data[5]));
			si->board_version = sysinfo_mkstr(dmi_string(dm, data[6]));
			si->board_serial = sysinfo_mkstr(dmi_string(dm, data[7]));
			break;
			
		case 3:
			si->chassis = sysinfo_mkstr(dmi_chassis_type(data[5]));
			break;
			
		}
		data+=dm->length;
		while(*data || data[1])
			data++;
		data+=2;
		i++;
	}
	free(buf);
}



int get_dmi_info(sysinfo_t *si)
/* reads some of the DMI info and puts it into si */
/* 0: error, else : ok */
{
	unsigned char buf[20];
	int fd=open("/dev/mem", O_RDONLY);
	long fp=0xE0000L;

	si->sys_vendor = NULL;
	si->sys_product = NULL;
	si->sys_serial = NULL;
	si->bios_vendor = NULL;
	si->bios_version = NULL;
	si->bios_release = NULL;
	si->board_vendor = NULL;
	si->board_product = NULL;
	si->board_version = NULL;
	si->board_serial = NULL;
	si->chassis = NULL;

	if(fd==-1)
	{
        	perror("/dev/mem");
		return 0;
	}
	if(lseek(fd,fp,0)==-1)
	{
		perror("seek");
		return 0;
	}
	
	fp -= 16;
	
	while( fp < 0xFFFFF)
	{
		fp+=16;
		if(read(fd, buf, 16)!=16)
                {
			perror("read");
                        return 0;
                }
		if(memcmp(buf, "_DMI_", 5)==0)
		{
			u16 num=buf[13]<<8|buf[12];
			u16 len=buf[7]<<8|buf[6];
			u32 base=buf[11]<<24|buf[10]<<16|buf[9]<<8|buf[8];
			dmi_table(fd, base,len, num, si);
		}
	}
	close(fd);
	return 1;
} /* get_dmi_info() */



void get_basic_sysinfo(sysinfo_t *si)
/* reads some sys info from /proc/cpuinfo and puts it into si */
/* A value of 0 or NULL means "could not be read" */
{
  FILE *f;
  char str[81];
  char *c;
  struct stat st;
  unsigned int i = 0;

  si->cpu = NULL;
  si->mhz = 0;
  si->mem = 0;
  memset(str, 0, 81);
  f = fopen("/proc/cpuinfo", "r");
  if (f)
  {
    while (fgets(str, 80, f))
    {
      if (0 == strncmp(str, "model name", 9))
      {
        c = str;
        while ( (*c != ':') && (*c != '\0') ) c++;
        if (':' == *c)
        {
          c++;
          while ( (' ' == *c) && (*c != '\0') ) c++;
          if (*c) si->cpu = sysinfo_mkstr(c);
        }
      }
      else if (0 == strncmp(str, "cpu MHz", 7))
      {
        c = str;
        while ( (*c != ':') && (*c != '\0') ) c++;
        if (':' == *c)
        {
          c++;
          while ( (!isdigit(*c)) && (*c != '\0') ) c++;
          if (*c) si->mhz = atoi(c);
        }
      }
    } /* while fgets() */
    fclose(f);
  } /* if (f) */

  if (stat("/proc/kcore", &st) != -1) si->mem = st.st_size;      /* low mem */
  f = fopen("/proc/meminfo", "r");
  if (f)
  {
    while (fgets(str, 80, f))
    {
      if (0 == strncmp(str, "HighTotal", 9))
      {
        c = str;
        while ( (*c != ':') && (*c != '\0') ) c++;
        if (':' == *c)
        {
          c++;
          while ( (' ' == *c) && (*c != '\0') ) c++;
          if (*c) i = atoi(c) * 1024;
          si->mem += i;                                          /* high mem */
        }
      }
    } /* while fgets() */
    fclose(f);
  } /* if (f) */
} /* get_basic_sysinfo() */



int ask_proxy(char **proxy)
/* asks for proxy settings and stores them in proxy[] */
/* memory for proxy[] is allocated */
/* 0: no proxy, 1: proxy stored in *proxy */
{
  char buf[256];
  unsigned char *c;

  clear();
#ifdef TEST
  printf("TEST");
#endif
  printf(" %s%s%s\n\n", BOLD, VERSION, NORMAL);
  printf("Falls Sie zum Zugriff auf das Internet einen Proxy-Server verwenden,\n");
  printf("geben Sie bitte seinen Namen in der Form\n");
  printf("\n  http://Proxy-Server:Port\n\n");
  printf("an, beispielsweise\n");
  printf("\n  %shttp://proxy.domain.de:8080%s\n\n", BOLD, NORMAL);
  printf("Wenn Sie keinen Proxy bentigen oder nicht auf das Internet zugreifen wollen,\n");
  printf("drcken Sie einfach Return.\n");
  printf("\n  %s                        %s\r  ", UNDERLINE, NORMAL);
  fflush(stdout);

  do
  {
    fgets(buf, 255, stdin);
    c = buf;
    while (*c > '\0')
    {
      if ('\n' == *c) *c = '\0';
      else if (*c < 32) *c = ' ';

      switch (*c)             /* check for illegal characters */
      {                       /* we must allow a lot of URL invalid chars */
        case '\t':            /* for username:password */
        case ' ':
        case '|':
        case '<':
        case '>':
        case '`':
        case '$':
        case '{':
        case '}':
        case '(':
        case ')':
                  c = NULL;
                  break;
        default:
                  c++;
      }
      if (!c)
      {
        printf("%s\nDie angegebene URL enthlt ungltige Zeichen.\n", NORMAL);
        printf("Bitte versuchen Sie es noch einmal:\n");
        printf("\n  %s                    %s\r  ", UNDERLINE, NORMAL);
        fflush(stdout);
        break;
      }
    }
  } while (!c);
  printf("%s", NORMAL); fflush(stdout);
  if ('\0' == buf[7]) return 0;
  *proxy = malloc(strlen(buf)+1);
  sprintf(*proxy, "%s", buf);
  return 1;
} /* ask_proxy() */



void get_netinfo(network_t *network)
/* tests via ping if network is up and name resolution works */
/* put the results into global variable network */
/* Called from display_sysinfo() so "Systeminformationen->System" should */
/* always display the current status */
{
  FILE *p;
  char str[81];
  char *c, *c1;

  network->up = 0;
  memset(network->device, 0, 8);
  memset(network->ip, 0, 16);
  memset(str, 0, 81);

  p = popen("/sbin/route -n", "r");          /* find default route */
  if (p)
  {
    while (fgets(str, 80, p))
    {
      if (!strncmp(str, "0.0.0.0", 7))
      {
        network->up = 1;
        c = str;
        while (*c > '\0') c++;
        while (*c != ' ')
        {
          if ('\n' == *c) *c = '\0';
          c--;
        }
        strncpy(network->device, c, 7);      /* default device */
      }
    }
    pclose(p);
  }

  if (network->device[0])
  {
    sprintf(str, "/sbin/ifconfig %s", network->device);
    p = popen(str, "r");      /* find IP adress of device */
    if (p)
    {
      if (fgets(str, 80, p))
      {
        if (fgets(str, 80, p))
        {
          c = str;
          while (*c > '\0')
          {
            if (':' == *c) break;
            c++;
          }
          c++;
          c1 = c;
          while (*c1 > '\0')
          {
            if (' ' == *c1) break;
            c1++;
          }
          *c1 = '\0';
          strncpy(network->ip, c, 15);
        }
      }
      pclose(p);
    } /* if (p) */
  }
} /* get_netinfo() */



int read_partitions(disk_t *disk)
/* reads partitions on disk disk->name; stores their data in corresponding */
/* entries in disk->first */
/* -1: error, else: # of partitions */
{
  char fdisk[80];
  FILE *p;
  char str[161];
  char *lines[1000];          /* 1000 partitions should be more than enough */
  char *c;
  partition_t *part = NULL;
  int i, n_part, n_lines;

  memset(fdisk, 0, 80);
  memset(str, 0, 161);
  if (!strncmp("RAID", disk->media, strlen(disk->media)))
   sprintf(fdisk, "/sbin/fdisk -l -u /dev/mapper/%s", disk->name);
  else sprintf(fdisk, "/sbin/fdisk -l -u /dev/%s", disk->name);

  p = popen(fdisk, "r");
  if (p)
  {
    n_lines = 0;
    while (fgets(str, 160, p))               /* read partition table */
    {
      if (0 == strncmp("/dev/", str, 5))     /* partition entry */
      {
        c = str;
        c += 5;                              /* skip "/dev/" */
        if (0 == strncmp("mapper/", c, 7)) c+= 7;      /* skip "mapper/" */
        lines[n_lines] = mkstr(c);
        n_lines++;
      }
    }
    pclose(p);

    n_part = 0;
    part = NULL;
    for (i=0; i<n_lines; i++)
    {
      if (disk->first)
      {
        part->next = malloc(sizeof(partition_t));
        memset(part->next, 0, sizeof(partition_t));
        part = part->next;
      }
      else                              /* first partition */
      {
        disk->first = malloc(sizeof(partition_t));
        memset(disk->first, 0, sizeof(partition_t));
        part = disk->first;
      }

      c = lines[i];
      while (*c != ' ') c++;
      *c = '\0';
      part->name = mkstr(lines[i]);          /* name of device */
      *c = ' ';

      while (*c == ' ') c++;
      if ('\0' == *c) continue;
      if ('*' == *c)                         /* "bootable" flag */
      {
        part->bootable = 1;
        c++;
        while (*c == ' ') c++;
        if ('\0' == *c) continue;
      }
      else part->bootable = 0;

      if (isdigit(*c))                       /* start sector */
       part->start = atoi(c);
      else continue;
      while (isdigit(*c)) c++;
      while (*c == ' ') c++;
      if ('\0' == *c) continue;
      if (isdigit(*c))                       /* end sector */
       part->end = atoi(c);
      else continue;
      while (isdigit(*c)) c++;

      while (*c == ' ') c++;
      if ('\0' == *c) continue;
      if (isdigit(*c))                       /* # blocks */
       part->blocks = atoi(c);
      else continue;
      while (isdigit(*c)) c++;
      if ('+' == *c) c++;

      while (*c == ' ') c++;
      if ('\0' == *c) continue;
      if (isxdigit(*c))                      /* type ID */
       part->id = strtol(c, NULL, 16);
      else continue;
      while (isxdigit(*c)) c++;

      while (*c == ' ') c++;
      if ('\0' == *c) continue;
      part->system = mkstr(c);               /* partition type name */
      if (!part->system) part->system = mkstr("???");
      if ('\0' == part->system[0]) part->system = mkstr("???");

      n_part++;
    } /* for i */
    return n_part;
  } /* if (p) */
  else return -1;

} /* read_partitions() */



int get_raid_info(disk_t *disk[])
/* checks if IDE RAID devices exist, determines their stripes and partitions, */
/* and puts them into disk[] */
/* 0: no IDE RAID found, else: # of IDE RAIDs */
{
  DIR *dir;
  struct dirent *entry;
  struct stat st;
  FILE *p;
  char *device[MAX_PARTITIONS];         /* list of block devices */
  int n_dev = 0;
  char *buf[MAX_PARTITIONS];            /* dmraid -r output */
  int n_lines;                          /* # lines in buf */
  char *c, *cc;
  int i, j, strl;
  int n_disk = 0;
  char str[PATH_MAX];
  
  p = popen("/usr/sbin/dmraid -r 2> /dev/null", "r");     /* get dmraid output into buf[] */
  if (!p) return 0;
  n_lines = 0;
  memset(str, 0, PATH_MAX);
  while (fgets(str, PATH_MAX-1, p))
  {
    c = str;
    while (*c > '\0')
    {
      if (!strncmp(c, " ok, ", 5))
/* valid line of form */
/* /dev/sdb: hpt45x, "hpt45x_cbijgijehg", stripe, ok, 160086400 sectors, data@ 0*/
      {
        strl = strlen(str);
        buf[n_lines] = malloc(strl);
        memset(buf[n_lines], 0, strl);
        strncpy(buf[n_lines], str, strl-1);    /* remove \n at end */
        n_lines++;
        break;
      }
      c++;
    }
  }
  pclose(p);

  dir = opendir("/dev/mapper");         /* check content of /dev/mapper; */
  if (!dir) return 0;               /* block devs are stored in device[] */
  entry = readdir(dir);
  while (entry)
  {
    sprintf(str, "/dev/mapper/%s", entry->d_name);
    if (0 == stat(str, &st))
    {
      if (S_ISBLK(st.st_mode))
      {
        device[n_dev] = malloc(NAME_MAX+1);
        memset(device[n_dev], 0, NAME_MAX+1);
        strncpy(device[n_dev], entry->d_name, NAME_MAX);
        n_dev++;
      }
    }
    entry = readdir(dir);
  }
  closedir(dir);
  if (!n_dev) return 0;                 /* no block devices in /dev/mapper */

  for (i=0; i<n_dev; i++)               /* find block devs in dmraid output */
  {
    strl = strlen(device[i]);
    disk[n_disk] = malloc(sizeof(disk_t));
    memset(disk[n_disk], 0, sizeof(disk_t));
    strncpy(disk[n_disk]->name, device[i], NAME_MAX);
    disk[n_disk]->media = mkstr("RAID");

    for (j=0; j<n_lines; j++)
    {
      c = buf[j];
      while (*(c+strl) > '\0')
      {
        if (!strncmp(c, device[i], strl))         /* matching dev in dmraid */
        {                                                         /* output */
          cc = buf[j];                            /* extract disk name */
          while ((*cc > '\0') && (*cc != ':')) cc++;
          if (*cc != ':') return 0;               /* format does not match */
          *cc = '\0';
          disk[n_disk]->stripe[disk[n_disk]->n_stripe] = mkstr(buf[j]+5);
                                                  /* remove /dev/ from start */
          disk[n_disk]->n_stripe++;
          *cc = ':';
          while (*cc > '\0')
          {
            if (!strncmp(cc, ", ok, ", 5))
            {
              cc += 5;
              while ((*cc < '0') || (*cc > '9')) cc++;
              disk[n_disk]->capacity += atoi(cc) / 2;       /* sectors, but */
              break;                                   /* we need 1k blocks */
            }
            cc++;
          }
          break;
        }
        c++;
      } /* while (*c+strl>\0) */
    } /* for j */

    if (0 == disk[n_disk]->n_stripe)    /* device not found in dmraid output */
     free(disk[n_disk]);
    else
    {
      disk[n_disk]->n_partitions = read_partitions(disk[n_disk]);
      if (-1 == disk[n_disk]->n_partitions)
      {
        disk[n_disk]->n_partitions = 0;
        disk[n_disk]->first = NULL;
        clear();
        printf("\nDie Partitionstabelle von %s lsst sich nicht lesen.\n",
         disk[n_disk]->name);
        printf("Dieser Datentrger kann daher nicht auf Viren untersucht werden.\n");
        wait_for_key();
      }
      n_disk++;
    }
  
  } /* for i */
  return n_disk;
} /* get_raid_info() */




int get_disk_info(disk_t *disk[], int no_raid)
/* gathers disk information in disk[] */
/* returns number of disks */
{
  FILE *f;
  unsigned int blocks;
  char str[81];
  char *c, *c1;
  int i, j = 0;
  int n_disk = 0;
  int n_raid = 0;


  char *prepare(char *str)
  /* skips major and minor block #, replaces '\n' by '\0' in str */
  /* NULL: invalid, else pointer in first char of "#_of_blocks name" */
  {
    char *c = str;
    while (*c)
    {
      if ('\n' == *c) *c = '\0';
      c++;
    }
    c = str;
    while (' ' == *c) c++;
    if (!isdigit(*c)) return NULL;      /* no major device # */
    while (isdigit(*c)) c++;
    while (' ' == *c) c++;
    if (!isdigit(*c)) return NULL;      /* no minor device # */
    while (isdigit(*c)) c++;
    while (' ' == *c) c++;
    if ('\0' == *c) return NULL;
    if (!isdigit(*c)) return NULL;      /* no # blocks */
    else return c;
  } /* prepare() */


  if (0 == no_raid) n_raid = get_raid_info(disk);
  n_disk = n_raid;
  f = fopen("/proc/partitions", "r");
  if (!f) return 0;

  while (fgets(str, 80, f))
  {
    c = prepare(str);
    if (!c) continue;                   /* invalid line */
    blocks = atoi(c);                   /* size in kByte */
    while (isdigit(*c)) c++;
    while (' ' == *c) c++;
    if ('\0' == *c) continue;           /* no device name */
    c1 = c;
    while (*c1 > ' ') c1++;             /* catch "hdc anything else" */
    *c1 = '\0';

    if (n_disk-n_raid)                  /* we already found a disk */
    {
      if (!strncmp(c, disk[n_disk-1]->name, strlen(disk[n_disk-1]->name)))
      {                                 /* a partition on the recent disk */
        continue;
      }
    }

    for (i=0; i<n_raid; i++)            /* disk used by RAID -> skip */
    {
      for (j=0; j<disk[i]->n_stripe; j++)
      {
        if (0 == strncmp(c, disk[i]->stripe[j], strlen(disk[i]->stripe[j])))
         goto skip;
      }
    }
    goto cont;
    skip: continue;
    cont:

    if ('d' == *(c+1))                            /* ?d? */
    {
      disk[n_disk] = malloc(sizeof(disk_t));
      memset(disk[n_disk], 0, sizeof(disk_t));
      strncpy(disk[n_disk]->name, c, 12);
      disk[n_disk]->capacity = blocks;
      
      if ('h' == *c)               /* IDE device */
      {
        FILE *f;
        sprintf(str, "/proc/ide/%s/media", c);
        f = fopen(str, "r");
        if (f)
        {
          if (fgets(str, 80, f)) disk[n_disk]->media = mkstr(str);
          else disk[n_disk]->media = mkstr("???");
          fclose(f);
        }
        else disk[n_disk]->media = mkstr("???");
        sprintf(str, "/proc/ide/%s/model", c);
        f = fopen(str, "r");
        if (f)
        {
          if (fgets(str, 80, f)) disk[n_disk]->model = mkstr(str);
          else disk[n_disk]->model = mkstr("???");
          fclose(f);
        }
        else disk[n_disk]->model = mkstr("???");
      }
      else
      {
        if ('s' == *c)                  /* SCSI device */
        {
          if ('d' == *(c+1))
           disk[n_disk]->media = mkstr("disk");
          else if ( ('c' == *(c+1)) && ('d' == *(c+2)) )
           disk[n_disk]->media = mkstr("cdrom");
          else disk[n_disk]->media = mkstr("???");
        }  
        else disk[n_disk]->media = mkstr("disk"); /* ataraid, something else */
      }
      n_disk++;
    }
  } /* while (fgets()) */
  fclose(f);

  for (i=n_raid; i<n_disk; i++)
  {
    if (0 == strncmp("disk", disk[i]->media, 4))
    {
      disk[i]->n_partitions = read_partitions(disk[i]);
      if (-1 == disk[i]->n_partitions)
      {
        disk[i]->n_partitions = 0;
        disk[i]->first = NULL;
        clear();
        printf("\nDie Partitionstabelle von %s lsst sich nicht lesen.\n",
         disk[n_disk]->name);
        printf("Dieser Datentrger kann daher nicht auf Viren untersucht werden.\n");
        wait_for_key();
      }
    }
  }
  return n_disk;
} /* get_disk_info() */  



void display_one_disk(disk_t *disk, int *lines)
/* display information on disk disk, with *lines lines already  on screen */
/* *lines is incremented */
{
  partition_t *part;
  int i;

  printf("%s%-20s %8s", BOLD, disk->name, disk->media);
  if (0 == strncmp("disk", disk->media, 4))
  {
    if (disk->capacity > 1024*1024)
     printf(" %6.1f GByte", disk->capacity / (1024.0*1024));
    else printf(" %6d MByte", disk->capacity / 1024);
  }
  else if (!strncmp("RAID", disk->media, 4))
  {
    if (disk->capacity > 1024*1024)
     printf(" %6.1f GByte  (", disk->capacity / (1024.0*1024));
    else printf(" %6d MByte  (", disk->capacity / 1024);
    for (i=0; i<disk->n_stripe; i++) printf("%s, ", disk->stripe[i]);
    printf("\b\b)");
  }
  else printf("    ---      ");
  printf("  %s%s\n", disk->model ? disk->model : "", NORMAL); /* no disk model */
  (*lines)++;

  if (disk->first)
  {
    part = disk->first;
    if (!part) printf("    Keine Partitionen gefunden\n");
    while (part)
    {
      for (i=0; i<N_CHECK; i++)
      {
        if (check[i] == part->id) 
        {
          printf(" %s->%s", BOLD, NORMAL);
          break;
        }
      }
      if (N_CHECK == i) printf("   ");

      printf(" %-23s  ", part->name);
      if (part->blocks > 1024*1024)
       printf("  %5.1f GByte", part->blocks / (1024.0*1024));
      else printf("  %5d MByte", part->blocks / 1024);
      printf("  (ID %02x, %s)\n", part->id, part->system);
      (*lines)++;
      if (*lines >= 23)
      {
        printf("%s--weiter--%s", REVERSE, NORMAL);
        fflush(stdout);
        (void) getchar();
        printf("\r");
        *lines=0;
      }
      part = part->next;
    } /* while (part) */
  }
} /* display_one_disk() */



void display_disk_info(disk_t *disk[], int n_disk)
/* displays disk and partition information for all disks */
{
  int i;
  int lines = 0;
  
  clear();
  printf("%s%s%s\n\n", BOLD, "Massenspeicher", NORMAL);
  lines+=2;
  if (n_disk) 
  {
    printf("%d Festplatte(n):\n", n_disk);
    lines++;
  }
  else printf("Keine Festplatten gefunden.\n\n");
  for (i=0; i<n_disk; i++) display_one_disk(disk[i], &lines);
  wait_for_key();
} /* display_disk_info() */



void display_sysinfo(sysinfo_t *sysinfo, network_t *network)
/* displays basic system information: CPU, memory, PCI devices */
{
  int i;
  int n = 0;
  FILE *p;
  char str[256];
  char *lspci[MAX_PCI];
  char *c;
  int lines=0;
  
  clear();
  printf("Bitte haben Sie einen Augenblick Geduld ... ");
  fflush(stdout);
  get_netinfo(network);
  clear();
  printf("%s%s%s\n\n", BOLD, "Systeminformationen", NORMAL);
  lines+=2;

  if (sysinfo->sys_vendor)
  {
    printf("%sComputer:%s %s %s%s%s%s%s%s\n", BOLD, NORMAL,
             sysinfo->sys_vendor, 
             sysinfo->sys_product ? sysinfo->sys_product : "",
             sysinfo->chassis ? " (" : "",
             sysinfo->chassis ? sysinfo->chassis : "",
             sysinfo->chassis ? ")" : "",
             sysinfo->sys_serial ? "\nSeriennummer: " : "",
             sysinfo->sys_serial ? sysinfo->sys_serial : "");
    lines++;
    if (sysinfo->sys_serial) lines++;
  }
  printf("%s, %d MHz, ",
           sysinfo->cpu ? sysinfo->cpu : "unbekannter CPU-Typ",
           sysinfo->mhz);
  if (sysinfo->mem)            /* value might be a little bit too small ... */
   printf("%u MByte\n", (sysinfo->mem + 400000) / (1024*1024));
  else printf("Speicherausbau nicht bestimmbar\n");
  lines++;
  printf("%sBoard:%s %s %s%s%s\n", BOLD, NORMAL,
              sysinfo->board_vendor, sysinfo->board_product,
              sysinfo->board_version ? "; Version: " : "",
              sysinfo->board_version ? sysinfo->board_version : "");
  lines++;
  if (sysinfo->bios_vendor || sysinfo->bios_version)
  {
    printf("%sBIOS:%s %s %s (%s)\n", BOLD, NORMAL,
               sysinfo->bios_vendor, sysinfo->bios_version, 
               sysinfo->bios_release);
    lines++;
  }

  printf("%sInternet-Anbindung:%s ", BOLD, NORMAL);
  if (network->up)
  {
    if (network->device[0]) 
    {
      printf("%s", network->device);
      if (network->ip) printf(" (IP-Adresse %s)\n", network->ip);
      else printf("\n");
    }
    else printf("Kein Netzwerk-Gert eingerichtet.\n");
  }
  else printf("nicht konfiguriert.\n");
  lines++;
  if (network->proxy) printf("Proxy: %s\n", network->proxy);
  lines++;

  printf("\n%sPCI-Gerte:%s\n", BOLD, NORMAL);
  lines+=2;
  p = popen("/sbin/lspci", "r");
  if (p)
  {
    n = 0;
    while (fgets(str, 255, p))
    {
      lspci[n] = mkstr(str);
      c = lspci[n];
      while (*c != '\0') c++;
      c--;
      if (')' == *c)
      {
        while ( (*c != '(') && (c > lspci[n]) ) c--;
        if ('(' == *c) *c = '\0';
      }
      n++;
      if (MAX_PCI-1 == n)
      {
        lspci[n] = mkstr("Weitere PCI-Gerte ...");
        break;
      }
    }
    pclose(p);

    for (i=0; i<n; i++)
    {
      printf("%s\n", lspci[i]);
      if (strlen(lspci[i]) <= 80) lines++;
      else lines+=2;
      if (lines >= 24)
      {
        printf("%s--weiter--%s", REVERSE, NORMAL);
        fflush(stdout);
        (void) getchar();
        printf("\r                \r");
        lines=0;
      }
    }
  } /* if (p) */

  else system("/sbin/lspci");
  wait_for_key();
} /* display_sysinfo() */

